% Analysis 
%
% Perform a Bayesian linear regression and estimate predictive intervals 
% for an MP-1 normal database.
%
% Copyright January, 2011, Columbia University, New York, NY. 
% All Rights Reserved 
%
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a commercial
% product is hereby granted without fee, provided that the above copyright
% notice appear in all copies and that both that copyright notice and this
% permission notice appear in supporting documentation, and that the name of
% Columbia University not be used in advertising or publicity pertaining to
% distribution of the software without specific, written prior permission. 
%
% COLUMBIA UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
% , INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL COLUMBIA UNIVERSITY BE LIABLE FOR 
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE.
%
% Author: Nichoas Bartlett bartlett@stat.columbia.edu

% read in all the data. each row of y constitutes the sensitivy measured
% for each of 50 patients for a particular location.  the ages of the 50
% patients are recorded in X.
[fullY, X] = ophthalmologyData;

% set the desired size of the posterior sample
samples = 101000;

% set the burn in
burnIn = 1000;

% indicate the number of predictions per sample from the posterior
predictionsPerSample = 10;

% create vector of ages for the desired prediction means and intervals
ages = 10:100;

% the number of visual field locations in the analysis
numSeries = 68;

% initialize a vector for the means
means = zeros(numSeries, length(ages));

% initialize vectors for prediction intervals
pi95  = zeros(2 * numSeries, length(ages));
pi98  = zeros(2 * numSeries, length(ages));
pi99  = zeros(2 * numSeries, length(ages));
pi995 = zeros(2 * numSeries, length(ages));

for series = 1:numSeries
    disp(['working on series ' num2str(series)]);

    % get data for given series
    Y = fullY(:,series);
    
    % run quick regression in order to initialize parameters
    r = regstats(Y,X,'linear',{'beta','covb','mse'});

    % initialize parameters
    alpha = r.beta(1);
    beta = r.beta(2);
    s2 = r.mse;
    
    % intialize matrix for posterior samples for this series
    posterior = zeros(samples,3);

    % draw posterior samples via Gibbs sampling for a given number of steps
    for sample = 1 : samples
        Y_complete = completeData(X,Y,alpha, beta, s2, 20);
        s2 = sampleSigma2(X,Y_complete,alpha, beta);
        [alpha beta] = sampleAlphaBeta(X,Y_complete,s2);

        posterior(sample,:) = [alpha beta s2];
    end

    % calculate expected values and insert in means matrix
    means(series,:) = mean(posterior((burnIn + 1):end,1)) + ages * mean(posterior((burnIn + 1):end,2));
    
    [p95,p98,p99,p995] = predictionIntervals(posterior((burnIn + 1):end,:), ages, predictionsPerSample);
    
    pi95([2 * series - 1 , 2 * series],:) = p95;
    pi98([2 * series - 1 , 2 * series],:)  = p98;
    pi99([2 * series - 1 , 2 * series],:) = p99;
    pi995([2 * series - 1 , 2 * series],:) = p995;

end

% the vectors starting with pi now contain the prediction intervals for 
% each series and the vector called means contains the predicted means for 
% each series.

clear X Y Y_complete alpha beta burnIn fullY numSeries p95 p98 p99 p995 ... 
    posterior predictionsPerSample r s2 sample samples series

% output to csv files
csvwrite('prediction_intervals_95.csv',pi95);
csvwrite('prediction_intervals_98.csv',pi98);
csvwrite('prediction_intervals_99.csv',pi99);
csvwrite('prediction_intervals_995.csv',pi995);

% an example of how to plot the outcome of this analysis for a given visual 
% field location is given below

% series = 1;
% 
% clf
% hold on
% % plots the mean line
% plot(ages, means(series,:));
% 
% % plots the prediction 95% confidence interval for the specified ages
% plot(ages, pi95(series * 2 - 1,:), 'r');
% plot(ages, pi95(series * 2 ,:), 'r');
